# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from datetime import date
from typing_extensions import Literal

import httpx

from ..types import (
    commitment_list_params,
    commitment_create_params,
    commitment_search_params,
    commitment_update_params,
)
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.commitment_response import CommitmentResponse
from ..types.commitment_fee_param import CommitmentFeeParam
from ..types.commitment_search_response import CommitmentSearchResponse

__all__ = ["CommitmentsResource", "AsyncCommitmentsResource"]


class CommitmentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CommitmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CommitmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CommitmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return CommitmentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        amount: float,
        currency: str,
        end_date: Union[str, date],
        start_date: Union[str, date],
        accounting_product_id: str | NotGiven = NOT_GIVEN,
        amount_first_bill: float | NotGiven = NOT_GIVEN,
        amount_pre_paid: float | NotGiven = NOT_GIVEN,
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        billing_interval: int | NotGiven = NOT_GIVEN,
        billing_offset: int | NotGiven = NOT_GIVEN,
        billing_plan_id: str | NotGiven = NOT_GIVEN,
        child_billing_mode: Literal["PARENT_SUMMARY", "PARENT_BREAKDOWN", "CHILD"] | NotGiven = NOT_GIVEN,
        commitment_fee_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        commitment_fee_description: str | NotGiven = NOT_GIVEN,
        commitment_usage_description: str | NotGiven = NOT_GIVEN,
        contract_id: str | NotGiven = NOT_GIVEN,
        drawdowns_accounting_product_id: str | NotGiven = NOT_GIVEN,
        fee_dates: Iterable[CommitmentFeeParam] | NotGiven = NOT_GIVEN,
        fees_accounting_product_id: str | NotGiven = NOT_GIVEN,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE", "USAGE", "MINIMUM_SPEND", "COUNTER_RUNNING_TOTAL_CHARGE", "COUNTER_ADJUSTMENT_DEBIT"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        overage_description: str | NotGiven = NOT_GIVEN,
        overage_surcharge_percent: float | NotGiven = NOT_GIVEN,
        product_ids: List[str] | NotGiven = NOT_GIVEN,
        separate_overage_usage: bool | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitmentResponse:
        """Create a new Commitment.

        Creates a new Commitment for an Organization.

        The request body must include all
        the necessary details such as the agreed amount, overage surcharge percentage,
        and the associated account and product details.

        **Note:** If some of the agreed Commitment amount remains unpaid at the start of
        an end-customer contract period, when you create a Commitment for an Account you
        can set up billing for the outstanding amount in one of two ways:

        - Select a Product _Plan to bill with_. Use the `billingPlanId` request
          parameter to select the Plan used for billing.
        - Define a _schedule of billing dates_. Omit a `billingPlanId` and use the
          `feeDates` request parameter to define a precise schedule of bill dates and
          amounts.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account the Commitment is
              added to.

          amount: The total amount that the customer has committed to pay.

          currency: The currency used for the Commitment. For example: USD.

          end_date: The end date of the Commitment period in ISO-8601 format.

              **Note:** End date is exclusive - if you set an end date of June 1st 2022, then
              the Commitment ceases to be active for the Account at midnight on May 31st 2022,
              and any Prepayment fees due are calculated up to that point in time, NOT up to
              midnight on June 1st

          start_date: The start date of the Commitment period in ISO-8601 format.

          accounting_product_id: The unique identifier (UUID) for the Product linked to the Commitment for
              accounting purposes. _(Optional)_

          amount_first_bill: The amount to be billed in the first invoice.

          amount_pre_paid: The amount that the customer has already paid upfront at the start of the
              Commitment service period.

          bill_epoch: The starting date _(in ISO-8601 date format)_ from which the billing cycles are
              calculated.

          billing_interval: How often the Commitment fees are applied to bills. For example, if the plan
              being used to bill for Commitment fees is set to issue bills every three months
              and the `billingInterval` is set to 2, then the Commitment fees are applied
              every six months.

          billing_offset: Defines an offset for when the Commitment fees are first applied to bills on the
              Account. For example, if bills are issued every three months and the
              `billingOffset` is 0, then the charge is applied to the first bill (at three
              months); if set to 1, it's applied to the next bill (at six months), and so on.

          billing_plan_id: The unique identifier (UUID) for the Product Plan used for billing Commitment
              fees due.

          child_billing_mode: If the Account is either a Parent or a Child Account, this specifies the Account
              hierarchy billing mode. The mode determines how billing will be handled and
              shown on bills for charges due on the Parent Account, and charges due on Child
              Accounts:

              - **Parent Breakdown** - a separate bill line item per Account. Default setting.

              - **Parent Summary** - single bill line item for all Accounts.

              - **Child** - the Child Account is billed.

          commitment_fee_bill_in_advance: A boolean value indicating whether the Commitment fee is billed in advance
              _(start of each billing period)_ or arrears _(end of each billing period)_.

              If no value is supplied, then the Organization Configuration value is used.

              - **TRUE** - bill in advance _(start of each billing period)_.
              - **FALSE** - bill in arrears _(end of each billing period)_.

          commitment_fee_description: A textual description of the Commitment fee.

          commitment_usage_description: A textual description of the Commitment usage.

          contract_id: The unique identifier (UUID) for a Contract you've created for the Account -
              used to add the Commitment to this Contract.

              **Note:** If you associate the Commitment with a Contract you must ensure the
              Account Plan attached to the Account has the same Contract associated with it.
              If the Account Plan Contract and Commitment Contract do not match, then at
              billing the Commitment amount will not be drawn-down against.

          drawdowns_accounting_product_id: Optional Product ID this Commitment consumptions should be attributed to for
              accounting purposes

          fee_dates: Used for billing any outstanding Commitment fees _on a schedule_.

              Create an array to define a series of bill dates and amounts covering specified
              service periods:

              - `date` - the billing date _(in ISO-8601 format)_.
              - `amount` - the billed amount.
              - `servicePeriodStartDate` and `servicePeriodEndDate` - defines the service
                period the bill covers _(in ISO-8601 format)_.

              **Notes:**

              - If you try to set `servicePeriodStartDate` _after_ `servicePeriodEndDate`,
                you'll receive an error.
              - You can set `servicePeriodStartDate` and `servicePeriodEndDate` to the _same
                date_ without receiving an error, but _please be sure_ your Commitment billing
                use case requires this.

          fees_accounting_product_id: Optional Product ID this Commitment fees should be attributed to for accounting
              purposes

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Commitment amount. Options are:

              - `MINIMUM_SPEND`
              - `STANDING_CHARGE`
              - `USAGE`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Commitment amount at billing.

          overage_description: A textual description of the overage charges.

          overage_surcharge_percent: The percentage surcharge applied to usage charges that exceed the Commitment
              amount.

              **Note:** You can enter a _negative percentage_ if you want to give a discount
              rate for usage to end customers who exceed their Commitment amount

          product_ids: A list of unique identifiers (UUIDs) for Products the Account consumes. Charges
              due for these Products will be made available for draw-down against the
              Commitment.

              **Note:** If not used, then charges due for all Products the Account consumes
              will be made available for draw-down against the Commitment.

          separate_overage_usage: A boolean value indicating whether the overage usage is billed separately or
              together. If overage usage is separated and a Commitment amount has been
              consumed by an Account, any subsequent line items on Bills against the Account
              for usage will show as separate "overage usage" charges, not simply as "usage"
              charges:

              - **TRUE** - billed separately.
              - **FALSE** - billed together.

              **Notes:**

              - Can be used only if no value or 0 has been defined for the
                `overageSurchargePercent` parameter. If you try to separate overage usage when
                a value other than 0 has been defined for `overageSurchargePercent`, you'll
                receive an error.
              - If a priced Plan is used to bill any outstanding Commitment fees due and the
                Plan is set up with overage pricing on a _tiered pricing structure_ and you
                enable separate bill line items for overage usage, then overage usage charges
                will be rated according to the overage pricing defined for the tiered pricing
                on the Plan.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/commitments",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "amount": amount,
                    "currency": currency,
                    "end_date": end_date,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "amount_first_bill": amount_first_bill,
                    "amount_pre_paid": amount_pre_paid,
                    "bill_epoch": bill_epoch,
                    "billing_interval": billing_interval,
                    "billing_offset": billing_offset,
                    "billing_plan_id": billing_plan_id,
                    "child_billing_mode": child_billing_mode,
                    "commitment_fee_bill_in_advance": commitment_fee_bill_in_advance,
                    "commitment_fee_description": commitment_fee_description,
                    "commitment_usage_description": commitment_usage_description,
                    "contract_id": contract_id,
                    "drawdowns_accounting_product_id": drawdowns_accounting_product_id,
                    "fee_dates": fee_dates,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "separate_overage_usage": separate_overage_usage,
                    "version": version,
                },
                commitment_create_params.CommitmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitmentResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitmentResponse:
        """
        Retrieve a specific Commitment.

        Retrieve the details of the Commitment with the given UUID. It provides
        comprehensive information about the Commitment, such as the agreed amount,
        overage surcharge percentage, and other related details.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/commitments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitmentResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        amount: float,
        currency: str,
        end_date: Union[str, date],
        start_date: Union[str, date],
        accounting_product_id: str | NotGiven = NOT_GIVEN,
        amount_first_bill: float | NotGiven = NOT_GIVEN,
        amount_pre_paid: float | NotGiven = NOT_GIVEN,
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        billing_interval: int | NotGiven = NOT_GIVEN,
        billing_offset: int | NotGiven = NOT_GIVEN,
        billing_plan_id: str | NotGiven = NOT_GIVEN,
        child_billing_mode: Literal["PARENT_SUMMARY", "PARENT_BREAKDOWN", "CHILD"] | NotGiven = NOT_GIVEN,
        commitment_fee_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        commitment_fee_description: str | NotGiven = NOT_GIVEN,
        commitment_usage_description: str | NotGiven = NOT_GIVEN,
        contract_id: str | NotGiven = NOT_GIVEN,
        drawdowns_accounting_product_id: str | NotGiven = NOT_GIVEN,
        fee_dates: Iterable[CommitmentFeeParam] | NotGiven = NOT_GIVEN,
        fees_accounting_product_id: str | NotGiven = NOT_GIVEN,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE", "USAGE", "MINIMUM_SPEND", "COUNTER_RUNNING_TOTAL_CHARGE", "COUNTER_ADJUSTMENT_DEBIT"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        overage_description: str | NotGiven = NOT_GIVEN,
        overage_surcharge_percent: float | NotGiven = NOT_GIVEN,
        product_ids: List[str] | NotGiven = NOT_GIVEN,
        separate_overage_usage: bool | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitmentResponse:
        """
        Modify a specific Commitment.

        Update the details of the Commitment with the given UUID. Use this endpoint to
        adjust Commitment parameters such as the fixed amount, overage surcharge
        percentage, or associated contract details.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account the Commitment is
              added to.

          amount: The total amount that the customer has committed to pay.

          currency: The currency used for the Commitment. For example: USD.

          end_date: The end date of the Commitment period in ISO-8601 format.

              **Note:** End date is exclusive - if you set an end date of June 1st 2022, then
              the Commitment ceases to be active for the Account at midnight on May 31st 2022,
              and any Prepayment fees due are calculated up to that point in time, NOT up to
              midnight on June 1st

          start_date: The start date of the Commitment period in ISO-8601 format.

          accounting_product_id: The unique identifier (UUID) for the Product linked to the Commitment for
              accounting purposes. _(Optional)_

          amount_first_bill: The amount to be billed in the first invoice.

          amount_pre_paid: The amount that the customer has already paid upfront at the start of the
              Commitment service period.

          bill_epoch: The starting date _(in ISO-8601 date format)_ from which the billing cycles are
              calculated.

          billing_interval: How often the Commitment fees are applied to bills. For example, if the plan
              being used to bill for Commitment fees is set to issue bills every three months
              and the `billingInterval` is set to 2, then the Commitment fees are applied
              every six months.

          billing_offset: Defines an offset for when the Commitment fees are first applied to bills on the
              Account. For example, if bills are issued every three months and the
              `billingOffset` is 0, then the charge is applied to the first bill (at three
              months); if set to 1, it's applied to the next bill (at six months), and so on.

          billing_plan_id: The unique identifier (UUID) for the Product Plan used for billing Commitment
              fees due.

          child_billing_mode: If the Account is either a Parent or a Child Account, this specifies the Account
              hierarchy billing mode. The mode determines how billing will be handled and
              shown on bills for charges due on the Parent Account, and charges due on Child
              Accounts:

              - **Parent Breakdown** - a separate bill line item per Account. Default setting.

              - **Parent Summary** - single bill line item for all Accounts.

              - **Child** - the Child Account is billed.

          commitment_fee_bill_in_advance: A boolean value indicating whether the Commitment fee is billed in advance
              _(start of each billing period)_ or arrears _(end of each billing period)_.

              If no value is supplied, then the Organization Configuration value is used.

              - **TRUE** - bill in advance _(start of each billing period)_.
              - **FALSE** - bill in arrears _(end of each billing period)_.

          commitment_fee_description: A textual description of the Commitment fee.

          commitment_usage_description: A textual description of the Commitment usage.

          contract_id: The unique identifier (UUID) for a Contract you've created for the Account -
              used to add the Commitment to this Contract.

              **Note:** If you associate the Commitment with a Contract you must ensure the
              Account Plan attached to the Account has the same Contract associated with it.
              If the Account Plan Contract and Commitment Contract do not match, then at
              billing the Commitment amount will not be drawn-down against.

          drawdowns_accounting_product_id: Optional Product ID this Commitment consumptions should be attributed to for
              accounting purposes

          fee_dates: Used for billing any outstanding Commitment fees _on a schedule_.

              Create an array to define a series of bill dates and amounts covering specified
              service periods:

              - `date` - the billing date _(in ISO-8601 format)_.
              - `amount` - the billed amount.
              - `servicePeriodStartDate` and `servicePeriodEndDate` - defines the service
                period the bill covers _(in ISO-8601 format)_.

              **Notes:**

              - If you try to set `servicePeriodStartDate` _after_ `servicePeriodEndDate`,
                you'll receive an error.
              - You can set `servicePeriodStartDate` and `servicePeriodEndDate` to the _same
                date_ without receiving an error, but _please be sure_ your Commitment billing
                use case requires this.

          fees_accounting_product_id: Optional Product ID this Commitment fees should be attributed to for accounting
              purposes

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Commitment amount. Options are:

              - `MINIMUM_SPEND`
              - `STANDING_CHARGE`
              - `USAGE`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Commitment amount at billing.

          overage_description: A textual description of the overage charges.

          overage_surcharge_percent: The percentage surcharge applied to usage charges that exceed the Commitment
              amount.

              **Note:** You can enter a _negative percentage_ if you want to give a discount
              rate for usage to end customers who exceed their Commitment amount

          product_ids: A list of unique identifiers (UUIDs) for Products the Account consumes. Charges
              due for these Products will be made available for draw-down against the
              Commitment.

              **Note:** If not used, then charges due for all Products the Account consumes
              will be made available for draw-down against the Commitment.

          separate_overage_usage: A boolean value indicating whether the overage usage is billed separately or
              together. If overage usage is separated and a Commitment amount has been
              consumed by an Account, any subsequent line items on Bills against the Account
              for usage will show as separate "overage usage" charges, not simply as "usage"
              charges:

              - **TRUE** - billed separately.
              - **FALSE** - billed together.

              **Notes:**

              - Can be used only if no value or 0 has been defined for the
                `overageSurchargePercent` parameter. If you try to separate overage usage when
                a value other than 0 has been defined for `overageSurchargePercent`, you'll
                receive an error.
              - If a priced Plan is used to bill any outstanding Commitment fees due and the
                Plan is set up with overage pricing on a _tiered pricing structure_ and you
                enable separate bill line items for overage usage, then overage usage charges
                will be rated according to the overage pricing defined for the tiered pricing
                on the Plan.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/commitments/{id}",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "amount": amount,
                    "currency": currency,
                    "end_date": end_date,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "amount_first_bill": amount_first_bill,
                    "amount_pre_paid": amount_pre_paid,
                    "bill_epoch": bill_epoch,
                    "billing_interval": billing_interval,
                    "billing_offset": billing_offset,
                    "billing_plan_id": billing_plan_id,
                    "child_billing_mode": child_billing_mode,
                    "commitment_fee_bill_in_advance": commitment_fee_bill_in_advance,
                    "commitment_fee_description": commitment_fee_description,
                    "commitment_usage_description": commitment_usage_description,
                    "contract_id": contract_id,
                    "drawdowns_accounting_product_id": drawdowns_accounting_product_id,
                    "fee_dates": fee_dates,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "separate_overage_usage": separate_overage_usage,
                    "version": version,
                },
                commitment_update_params.CommitmentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitmentResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | NotGiven = NOT_GIVEN,
        contract_id: Optional[str] | NotGiven = NOT_GIVEN,
        date: str | NotGiven = NOT_GIVEN,
        end_date_end: str | NotGiven = NOT_GIVEN,
        end_date_start: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        product_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[CommitmentResponse]:
        """
        Retrieve a list of Commitments.

        Retrieves a list of all Commitments associated with an Organization. This
        endpoint supports pagination and includes various query parameters to filter the
        Commitments based on Account, Product, date, and end dates.

        Args:
          account_id: The unique identifier (UUID) for the Account. This parameter helps filter the
              Commitments related to a specific end-customer Account.

          date: A date _(in ISO-8601 format)_ to filter Commitments which are active on this
              specific date.

          end_date_end: A date _(in ISO-8601 format)_ used to filter Commitments. Only Commitments with
              end dates before this date will be included.

          end_date_start: A date _(in ISO-8601 format)_ used to filter Commitments. Only Commitments with
              end dates on or after this date will be included.

          ids: A list of unique identifiers (UUIDs) for the Commitments to retrieve. Use this
              to fetch specific Commitments in a single request.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Commitments in a paginated list.

          page_size: Specifies the maximum number of Commitments to retrieve per page.

          product_id: The unique identifier (UUID) for the Product. This parameter helps filter the
              Commitments related to a specific Product.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/commitments",
            page=SyncCursor[CommitmentResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "contract_id": contract_id,
                        "date": date,
                        "end_date_end": end_date_end,
                        "end_date_start": end_date_start,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "product_id": product_id,
                    },
                    commitment_list_params.CommitmentListParams,
                ),
            ),
            model=CommitmentResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitmentResponse:
        """Remove a specific Commitment.

        Deletes the Commitment with the given UUID.

        Use this endpoint when a Commitment
        is no longer valid or needs to be removed from the system.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/commitments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitmentResponse,
        )

    def search(
        self,
        *,
        org_id: str | None = None,
        from_document: int | NotGiven = NOT_GIVEN,
        operator: Literal["AND", "OR"] | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        search_query: str | NotGiven = NOT_GIVEN,
        sort_by: str | NotGiven = NOT_GIVEN,
        sort_order: Literal["ASC", "DESC"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitmentSearchResponse:
        """
        Search for commitment entities.

        This endpoint executes a search query for Commitments based on the user
        specified search criteria. The search query is customizable, allowing for
        complex nested conditions and sorting. The returned list of Commitments can be
        paginated for easier management.

        Args:
          from_document: `fromDocument` for multi page retrievals.

          operator: Search Operator to be used while querying search.

          page_size: Number of Commitments to retrieve per page.

              **NOTE:** If not defined, default is 10.

          search_query:
              Query for data using special syntax:

              - Query parameters should be delimited using $ (dollar sign).
              - Allowed comparators are:
                - (greater than) >
                - (greater than or equal to) >=
                - (equal to) :
                - (less than) <
                - (less than or equal to) <=
                - (match phrase/prefix) ~
              - Allowed parameters: startDate, endDate, contractId, accountId, productId,
                productIds, id, createdBy, dtCreated, lastModifiedBy, ids.
              - Query example:
                - searchQuery=startDate>2023-01-01$accountId:062085ab-a301-4f21-a081-411020864452.
                - This query is translated into: find commitments where the startDate is older
                  than 2023-01-01 AND the accountId is equal to
                  062085ab-a301-4f21-a081-411020864452.

              **Note:** Using the ~ match phrase/prefix comparator. For best results, we
              recommend treating this as a "starts with" comparator for your search query.

          sort_by: Name of the parameter on which sorting is performed. Use any field available on
              the Commitment entity to sort by, such as `accountId`, `endDate`, and so on.

          sort_order: Sorting order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get(
            f"/organizations/{org_id}/commitments/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "from_document": from_document,
                        "operator": operator,
                        "page_size": page_size,
                        "search_query": search_query,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    commitment_search_params.CommitmentSearchParams,
                ),
            ),
            cast_to=CommitmentSearchResponse,
        )


class AsyncCommitmentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCommitmentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCommitmentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCommitmentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncCommitmentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        amount: float,
        currency: str,
        end_date: Union[str, date],
        start_date: Union[str, date],
        accounting_product_id: str | NotGiven = NOT_GIVEN,
        amount_first_bill: float | NotGiven = NOT_GIVEN,
        amount_pre_paid: float | NotGiven = NOT_GIVEN,
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        billing_interval: int | NotGiven = NOT_GIVEN,
        billing_offset: int | NotGiven = NOT_GIVEN,
        billing_plan_id: str | NotGiven = NOT_GIVEN,
        child_billing_mode: Literal["PARENT_SUMMARY", "PARENT_BREAKDOWN", "CHILD"] | NotGiven = NOT_GIVEN,
        commitment_fee_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        commitment_fee_description: str | NotGiven = NOT_GIVEN,
        commitment_usage_description: str | NotGiven = NOT_GIVEN,
        contract_id: str | NotGiven = NOT_GIVEN,
        drawdowns_accounting_product_id: str | NotGiven = NOT_GIVEN,
        fee_dates: Iterable[CommitmentFeeParam] | NotGiven = NOT_GIVEN,
        fees_accounting_product_id: str | NotGiven = NOT_GIVEN,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE", "USAGE", "MINIMUM_SPEND", "COUNTER_RUNNING_TOTAL_CHARGE", "COUNTER_ADJUSTMENT_DEBIT"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        overage_description: str | NotGiven = NOT_GIVEN,
        overage_surcharge_percent: float | NotGiven = NOT_GIVEN,
        product_ids: List[str] | NotGiven = NOT_GIVEN,
        separate_overage_usage: bool | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitmentResponse:
        """Create a new Commitment.

        Creates a new Commitment for an Organization.

        The request body must include all
        the necessary details such as the agreed amount, overage surcharge percentage,
        and the associated account and product details.

        **Note:** If some of the agreed Commitment amount remains unpaid at the start of
        an end-customer contract period, when you create a Commitment for an Account you
        can set up billing for the outstanding amount in one of two ways:

        - Select a Product _Plan to bill with_. Use the `billingPlanId` request
          parameter to select the Plan used for billing.
        - Define a _schedule of billing dates_. Omit a `billingPlanId` and use the
          `feeDates` request parameter to define a precise schedule of bill dates and
          amounts.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account the Commitment is
              added to.

          amount: The total amount that the customer has committed to pay.

          currency: The currency used for the Commitment. For example: USD.

          end_date: The end date of the Commitment period in ISO-8601 format.

              **Note:** End date is exclusive - if you set an end date of June 1st 2022, then
              the Commitment ceases to be active for the Account at midnight on May 31st 2022,
              and any Prepayment fees due are calculated up to that point in time, NOT up to
              midnight on June 1st

          start_date: The start date of the Commitment period in ISO-8601 format.

          accounting_product_id: The unique identifier (UUID) for the Product linked to the Commitment for
              accounting purposes. _(Optional)_

          amount_first_bill: The amount to be billed in the first invoice.

          amount_pre_paid: The amount that the customer has already paid upfront at the start of the
              Commitment service period.

          bill_epoch: The starting date _(in ISO-8601 date format)_ from which the billing cycles are
              calculated.

          billing_interval: How often the Commitment fees are applied to bills. For example, if the plan
              being used to bill for Commitment fees is set to issue bills every three months
              and the `billingInterval` is set to 2, then the Commitment fees are applied
              every six months.

          billing_offset: Defines an offset for when the Commitment fees are first applied to bills on the
              Account. For example, if bills are issued every three months and the
              `billingOffset` is 0, then the charge is applied to the first bill (at three
              months); if set to 1, it's applied to the next bill (at six months), and so on.

          billing_plan_id: The unique identifier (UUID) for the Product Plan used for billing Commitment
              fees due.

          child_billing_mode: If the Account is either a Parent or a Child Account, this specifies the Account
              hierarchy billing mode. The mode determines how billing will be handled and
              shown on bills for charges due on the Parent Account, and charges due on Child
              Accounts:

              - **Parent Breakdown** - a separate bill line item per Account. Default setting.

              - **Parent Summary** - single bill line item for all Accounts.

              - **Child** - the Child Account is billed.

          commitment_fee_bill_in_advance: A boolean value indicating whether the Commitment fee is billed in advance
              _(start of each billing period)_ or arrears _(end of each billing period)_.

              If no value is supplied, then the Organization Configuration value is used.

              - **TRUE** - bill in advance _(start of each billing period)_.
              - **FALSE** - bill in arrears _(end of each billing period)_.

          commitment_fee_description: A textual description of the Commitment fee.

          commitment_usage_description: A textual description of the Commitment usage.

          contract_id: The unique identifier (UUID) for a Contract you've created for the Account -
              used to add the Commitment to this Contract.

              **Note:** If you associate the Commitment with a Contract you must ensure the
              Account Plan attached to the Account has the same Contract associated with it.
              If the Account Plan Contract and Commitment Contract do not match, then at
              billing the Commitment amount will not be drawn-down against.

          drawdowns_accounting_product_id: Optional Product ID this Commitment consumptions should be attributed to for
              accounting purposes

          fee_dates: Used for billing any outstanding Commitment fees _on a schedule_.

              Create an array to define a series of bill dates and amounts covering specified
              service periods:

              - `date` - the billing date _(in ISO-8601 format)_.
              - `amount` - the billed amount.
              - `servicePeriodStartDate` and `servicePeriodEndDate` - defines the service
                period the bill covers _(in ISO-8601 format)_.

              **Notes:**

              - If you try to set `servicePeriodStartDate` _after_ `servicePeriodEndDate`,
                you'll receive an error.
              - You can set `servicePeriodStartDate` and `servicePeriodEndDate` to the _same
                date_ without receiving an error, but _please be sure_ your Commitment billing
                use case requires this.

          fees_accounting_product_id: Optional Product ID this Commitment fees should be attributed to for accounting
              purposes

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Commitment amount. Options are:

              - `MINIMUM_SPEND`
              - `STANDING_CHARGE`
              - `USAGE`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Commitment amount at billing.

          overage_description: A textual description of the overage charges.

          overage_surcharge_percent: The percentage surcharge applied to usage charges that exceed the Commitment
              amount.

              **Note:** You can enter a _negative percentage_ if you want to give a discount
              rate for usage to end customers who exceed their Commitment amount

          product_ids: A list of unique identifiers (UUIDs) for Products the Account consumes. Charges
              due for these Products will be made available for draw-down against the
              Commitment.

              **Note:** If not used, then charges due for all Products the Account consumes
              will be made available for draw-down against the Commitment.

          separate_overage_usage: A boolean value indicating whether the overage usage is billed separately or
              together. If overage usage is separated and a Commitment amount has been
              consumed by an Account, any subsequent line items on Bills against the Account
              for usage will show as separate "overage usage" charges, not simply as "usage"
              charges:

              - **TRUE** - billed separately.
              - **FALSE** - billed together.

              **Notes:**

              - Can be used only if no value or 0 has been defined for the
                `overageSurchargePercent` parameter. If you try to separate overage usage when
                a value other than 0 has been defined for `overageSurchargePercent`, you'll
                receive an error.
              - If a priced Plan is used to bill any outstanding Commitment fees due and the
                Plan is set up with overage pricing on a _tiered pricing structure_ and you
                enable separate bill line items for overage usage, then overage usage charges
                will be rated according to the overage pricing defined for the tiered pricing
                on the Plan.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/commitments",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "amount": amount,
                    "currency": currency,
                    "end_date": end_date,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "amount_first_bill": amount_first_bill,
                    "amount_pre_paid": amount_pre_paid,
                    "bill_epoch": bill_epoch,
                    "billing_interval": billing_interval,
                    "billing_offset": billing_offset,
                    "billing_plan_id": billing_plan_id,
                    "child_billing_mode": child_billing_mode,
                    "commitment_fee_bill_in_advance": commitment_fee_bill_in_advance,
                    "commitment_fee_description": commitment_fee_description,
                    "commitment_usage_description": commitment_usage_description,
                    "contract_id": contract_id,
                    "drawdowns_accounting_product_id": drawdowns_accounting_product_id,
                    "fee_dates": fee_dates,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "separate_overage_usage": separate_overage_usage,
                    "version": version,
                },
                commitment_create_params.CommitmentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitmentResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitmentResponse:
        """
        Retrieve a specific Commitment.

        Retrieve the details of the Commitment with the given UUID. It provides
        comprehensive information about the Commitment, such as the agreed amount,
        overage surcharge percentage, and other related details.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/commitments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitmentResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        amount: float,
        currency: str,
        end_date: Union[str, date],
        start_date: Union[str, date],
        accounting_product_id: str | NotGiven = NOT_GIVEN,
        amount_first_bill: float | NotGiven = NOT_GIVEN,
        amount_pre_paid: float | NotGiven = NOT_GIVEN,
        bill_epoch: Union[str, date] | NotGiven = NOT_GIVEN,
        billing_interval: int | NotGiven = NOT_GIVEN,
        billing_offset: int | NotGiven = NOT_GIVEN,
        billing_plan_id: str | NotGiven = NOT_GIVEN,
        child_billing_mode: Literal["PARENT_SUMMARY", "PARENT_BREAKDOWN", "CHILD"] | NotGiven = NOT_GIVEN,
        commitment_fee_bill_in_advance: bool | NotGiven = NOT_GIVEN,
        commitment_fee_description: str | NotGiven = NOT_GIVEN,
        commitment_usage_description: str | NotGiven = NOT_GIVEN,
        contract_id: str | NotGiven = NOT_GIVEN,
        drawdowns_accounting_product_id: str | NotGiven = NOT_GIVEN,
        fee_dates: Iterable[CommitmentFeeParam] | NotGiven = NOT_GIVEN,
        fees_accounting_product_id: str | NotGiven = NOT_GIVEN,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE", "USAGE", "MINIMUM_SPEND", "COUNTER_RUNNING_TOTAL_CHARGE", "COUNTER_ADJUSTMENT_DEBIT"
            ]
        ]
        | NotGiven = NOT_GIVEN,
        overage_description: str | NotGiven = NOT_GIVEN,
        overage_surcharge_percent: float | NotGiven = NOT_GIVEN,
        product_ids: List[str] | NotGiven = NOT_GIVEN,
        separate_overage_usage: bool | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitmentResponse:
        """
        Modify a specific Commitment.

        Update the details of the Commitment with the given UUID. Use this endpoint to
        adjust Commitment parameters such as the fixed amount, overage surcharge
        percentage, or associated contract details.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account the Commitment is
              added to.

          amount: The total amount that the customer has committed to pay.

          currency: The currency used for the Commitment. For example: USD.

          end_date: The end date of the Commitment period in ISO-8601 format.

              **Note:** End date is exclusive - if you set an end date of June 1st 2022, then
              the Commitment ceases to be active for the Account at midnight on May 31st 2022,
              and any Prepayment fees due are calculated up to that point in time, NOT up to
              midnight on June 1st

          start_date: The start date of the Commitment period in ISO-8601 format.

          accounting_product_id: The unique identifier (UUID) for the Product linked to the Commitment for
              accounting purposes. _(Optional)_

          amount_first_bill: The amount to be billed in the first invoice.

          amount_pre_paid: The amount that the customer has already paid upfront at the start of the
              Commitment service period.

          bill_epoch: The starting date _(in ISO-8601 date format)_ from which the billing cycles are
              calculated.

          billing_interval: How often the Commitment fees are applied to bills. For example, if the plan
              being used to bill for Commitment fees is set to issue bills every three months
              and the `billingInterval` is set to 2, then the Commitment fees are applied
              every six months.

          billing_offset: Defines an offset for when the Commitment fees are first applied to bills on the
              Account. For example, if bills are issued every three months and the
              `billingOffset` is 0, then the charge is applied to the first bill (at three
              months); if set to 1, it's applied to the next bill (at six months), and so on.

          billing_plan_id: The unique identifier (UUID) for the Product Plan used for billing Commitment
              fees due.

          child_billing_mode: If the Account is either a Parent or a Child Account, this specifies the Account
              hierarchy billing mode. The mode determines how billing will be handled and
              shown on bills for charges due on the Parent Account, and charges due on Child
              Accounts:

              - **Parent Breakdown** - a separate bill line item per Account. Default setting.

              - **Parent Summary** - single bill line item for all Accounts.

              - **Child** - the Child Account is billed.

          commitment_fee_bill_in_advance: A boolean value indicating whether the Commitment fee is billed in advance
              _(start of each billing period)_ or arrears _(end of each billing period)_.

              If no value is supplied, then the Organization Configuration value is used.

              - **TRUE** - bill in advance _(start of each billing period)_.
              - **FALSE** - bill in arrears _(end of each billing period)_.

          commitment_fee_description: A textual description of the Commitment fee.

          commitment_usage_description: A textual description of the Commitment usage.

          contract_id: The unique identifier (UUID) for a Contract you've created for the Account -
              used to add the Commitment to this Contract.

              **Note:** If you associate the Commitment with a Contract you must ensure the
              Account Plan attached to the Account has the same Contract associated with it.
              If the Account Plan Contract and Commitment Contract do not match, then at
              billing the Commitment amount will not be drawn-down against.

          drawdowns_accounting_product_id: Optional Product ID this Commitment consumptions should be attributed to for
              accounting purposes

          fee_dates: Used for billing any outstanding Commitment fees _on a schedule_.

              Create an array to define a series of bill dates and amounts covering specified
              service periods:

              - `date` - the billing date _(in ISO-8601 format)_.
              - `amount` - the billed amount.
              - `servicePeriodStartDate` and `servicePeriodEndDate` - defines the service
                period the bill covers _(in ISO-8601 format)_.

              **Notes:**

              - If you try to set `servicePeriodStartDate` _after_ `servicePeriodEndDate`,
                you'll receive an error.
              - You can set `servicePeriodStartDate` and `servicePeriodEndDate` to the _same
                date_ without receiving an error, but _please be sure_ your Commitment billing
                use case requires this.

          fees_accounting_product_id: Optional Product ID this Commitment fees should be attributed to for accounting
              purposes

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Commitment amount. Options are:

              - `MINIMUM_SPEND`
              - `STANDING_CHARGE`
              - `USAGE`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Commitment amount at billing.

          overage_description: A textual description of the overage charges.

          overage_surcharge_percent: The percentage surcharge applied to usage charges that exceed the Commitment
              amount.

              **Note:** You can enter a _negative percentage_ if you want to give a discount
              rate for usage to end customers who exceed their Commitment amount

          product_ids: A list of unique identifiers (UUIDs) for Products the Account consumes. Charges
              due for these Products will be made available for draw-down against the
              Commitment.

              **Note:** If not used, then charges due for all Products the Account consumes
              will be made available for draw-down against the Commitment.

          separate_overage_usage: A boolean value indicating whether the overage usage is billed separately or
              together. If overage usage is separated and a Commitment amount has been
              consumed by an Account, any subsequent line items on Bills against the Account
              for usage will show as separate "overage usage" charges, not simply as "usage"
              charges:

              - **TRUE** - billed separately.
              - **FALSE** - billed together.

              **Notes:**

              - Can be used only if no value or 0 has been defined for the
                `overageSurchargePercent` parameter. If you try to separate overage usage when
                a value other than 0 has been defined for `overageSurchargePercent`, you'll
                receive an error.
              - If a priced Plan is used to bill any outstanding Commitment fees due and the
                Plan is set up with overage pricing on a _tiered pricing structure_ and you
                enable separate bill line items for overage usage, then overage usage charges
                will be rated according to the overage pricing defined for the tiered pricing
                on the Plan.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/commitments/{id}",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "amount": amount,
                    "currency": currency,
                    "end_date": end_date,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "amount_first_bill": amount_first_bill,
                    "amount_pre_paid": amount_pre_paid,
                    "bill_epoch": bill_epoch,
                    "billing_interval": billing_interval,
                    "billing_offset": billing_offset,
                    "billing_plan_id": billing_plan_id,
                    "child_billing_mode": child_billing_mode,
                    "commitment_fee_bill_in_advance": commitment_fee_bill_in_advance,
                    "commitment_fee_description": commitment_fee_description,
                    "commitment_usage_description": commitment_usage_description,
                    "contract_id": contract_id,
                    "drawdowns_accounting_product_id": drawdowns_accounting_product_id,
                    "fee_dates": fee_dates,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "separate_overage_usage": separate_overage_usage,
                    "version": version,
                },
                commitment_update_params.CommitmentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitmentResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | NotGiven = NOT_GIVEN,
        contract_id: Optional[str] | NotGiven = NOT_GIVEN,
        date: str | NotGiven = NOT_GIVEN,
        end_date_end: str | NotGiven = NOT_GIVEN,
        end_date_start: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        product_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[CommitmentResponse, AsyncCursor[CommitmentResponse]]:
        """
        Retrieve a list of Commitments.

        Retrieves a list of all Commitments associated with an Organization. This
        endpoint supports pagination and includes various query parameters to filter the
        Commitments based on Account, Product, date, and end dates.

        Args:
          account_id: The unique identifier (UUID) for the Account. This parameter helps filter the
              Commitments related to a specific end-customer Account.

          date: A date _(in ISO-8601 format)_ to filter Commitments which are active on this
              specific date.

          end_date_end: A date _(in ISO-8601 format)_ used to filter Commitments. Only Commitments with
              end dates before this date will be included.

          end_date_start: A date _(in ISO-8601 format)_ used to filter Commitments. Only Commitments with
              end dates on or after this date will be included.

          ids: A list of unique identifiers (UUIDs) for the Commitments to retrieve. Use this
              to fetch specific Commitments in a single request.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Commitments in a paginated list.

          page_size: Specifies the maximum number of Commitments to retrieve per page.

          product_id: The unique identifier (UUID) for the Product. This parameter helps filter the
              Commitments related to a specific Product.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/commitments",
            page=AsyncCursor[CommitmentResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "contract_id": contract_id,
                        "date": date,
                        "end_date_end": end_date_end,
                        "end_date_start": end_date_start,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "product_id": product_id,
                    },
                    commitment_list_params.CommitmentListParams,
                ),
            ),
            model=CommitmentResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitmentResponse:
        """Remove a specific Commitment.

        Deletes the Commitment with the given UUID.

        Use this endpoint when a Commitment
        is no longer valid or needs to be removed from the system.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/commitments/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CommitmentResponse,
        )

    async def search(
        self,
        *,
        org_id: str | None = None,
        from_document: int | NotGiven = NOT_GIVEN,
        operator: Literal["AND", "OR"] | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        search_query: str | NotGiven = NOT_GIVEN,
        sort_by: str | NotGiven = NOT_GIVEN,
        sort_order: Literal["ASC", "DESC"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CommitmentSearchResponse:
        """
        Search for commitment entities.

        This endpoint executes a search query for Commitments based on the user
        specified search criteria. The search query is customizable, allowing for
        complex nested conditions and sorting. The returned list of Commitments can be
        paginated for easier management.

        Args:
          from_document: `fromDocument` for multi page retrievals.

          operator: Search Operator to be used while querying search.

          page_size: Number of Commitments to retrieve per page.

              **NOTE:** If not defined, default is 10.

          search_query:
              Query for data using special syntax:

              - Query parameters should be delimited using $ (dollar sign).
              - Allowed comparators are:
                - (greater than) >
                - (greater than or equal to) >=
                - (equal to) :
                - (less than) <
                - (less than or equal to) <=
                - (match phrase/prefix) ~
              - Allowed parameters: startDate, endDate, contractId, accountId, productId,
                productIds, id, createdBy, dtCreated, lastModifiedBy, ids.
              - Query example:
                - searchQuery=startDate>2023-01-01$accountId:062085ab-a301-4f21-a081-411020864452.
                - This query is translated into: find commitments where the startDate is older
                  than 2023-01-01 AND the accountId is equal to
                  062085ab-a301-4f21-a081-411020864452.

              **Note:** Using the ~ match phrase/prefix comparator. For best results, we
              recommend treating this as a "starts with" comparator for your search query.

          sort_by: Name of the parameter on which sorting is performed. Use any field available on
              the Commitment entity to sort by, such as `accountId`, `endDate`, and so on.

          sort_order: Sorting order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._get(
            f"/organizations/{org_id}/commitments/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "from_document": from_document,
                        "operator": operator,
                        "page_size": page_size,
                        "search_query": search_query,
                        "sort_by": sort_by,
                        "sort_order": sort_order,
                    },
                    commitment_search_params.CommitmentSearchParams,
                ),
            ),
            cast_to=CommitmentSearchResponse,
        )


class CommitmentsResourceWithRawResponse:
    def __init__(self, commitments: CommitmentsResource) -> None:
        self._commitments = commitments

        self.create = to_raw_response_wrapper(
            commitments.create,
        )
        self.retrieve = to_raw_response_wrapper(
            commitments.retrieve,
        )
        self.update = to_raw_response_wrapper(
            commitments.update,
        )
        self.list = to_raw_response_wrapper(
            commitments.list,
        )
        self.delete = to_raw_response_wrapper(
            commitments.delete,
        )
        self.search = to_raw_response_wrapper(
            commitments.search,
        )


class AsyncCommitmentsResourceWithRawResponse:
    def __init__(self, commitments: AsyncCommitmentsResource) -> None:
        self._commitments = commitments

        self.create = async_to_raw_response_wrapper(
            commitments.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            commitments.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            commitments.update,
        )
        self.list = async_to_raw_response_wrapper(
            commitments.list,
        )
        self.delete = async_to_raw_response_wrapper(
            commitments.delete,
        )
        self.search = async_to_raw_response_wrapper(
            commitments.search,
        )


class CommitmentsResourceWithStreamingResponse:
    def __init__(self, commitments: CommitmentsResource) -> None:
        self._commitments = commitments

        self.create = to_streamed_response_wrapper(
            commitments.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            commitments.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            commitments.update,
        )
        self.list = to_streamed_response_wrapper(
            commitments.list,
        )
        self.delete = to_streamed_response_wrapper(
            commitments.delete,
        )
        self.search = to_streamed_response_wrapper(
            commitments.search,
        )


class AsyncCommitmentsResourceWithStreamingResponse:
    def __init__(self, commitments: AsyncCommitmentsResource) -> None:
        self._commitments = commitments

        self.create = async_to_streamed_response_wrapper(
            commitments.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            commitments.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            commitments.update,
        )
        self.list = async_to_streamed_response_wrapper(
            commitments.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            commitments.delete,
        )
        self.search = async_to_streamed_response_wrapper(
            commitments.search,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .commitment_response import CommitmentResponse

__all__ = ["CommitmentSearchResponse"]


class CommitmentSearchResponse(BaseModel):
    data: Optional[List[CommitmentResponse]] = None
    """The list of Commitments information."""

    next_token: Optional[str] = FieldInfo(alias="nextToken", default=None)
    """The `nextToken` for multi-page retrievals.

    It is used to fetch the next page of Commitments in a paginated list.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union

import httpx

from ..types import custom_field_update_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.custom_fields_response import CustomFieldsResponse

__all__ = ["CustomFieldsResource", "AsyncCustomFieldsResource"]


class CustomFieldsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CustomFieldsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CustomFieldsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomFieldsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return CustomFieldsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CustomFieldsResponse:
        """
        Retrieve all Custom Fields added at Organizational level for the entities that
        support them.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get(
            f"/organizations/{org_id}/customfields",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomFieldsResponse,
        )

    def update(
        self,
        *,
        org_id: str | None = None,
        account: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        account_plan: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        aggregation: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        compound_aggregation: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        meter: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        organization: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        plan: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        plan_template: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        product: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CustomFieldsResponse:
        """
        Update Custom Fields added at Organization level to entities that support them.

        Args:
          account: Updates to Account entity CustomFields.

          account_plan: Updates to accountPlan entity CustomFields.

          aggregation: Updates to simple Aggregation entity CustomFields.

          compound_aggregation: Updates to Compound Aggregation entity CustomFields.

          meter: Updates to Meter entity CustomFields.

          organization: Updates to Organization CustomFields.

          plan: Updates to Plan entity CustomFields.

          plan_template: Updates to planTemplate entity CustomFields.

          product: Updates to Product entity CustomFields.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._put(
            f"/organizations/{org_id}/customfields",
            body=maybe_transform(
                {
                    "account": account,
                    "account_plan": account_plan,
                    "aggregation": aggregation,
                    "compound_aggregation": compound_aggregation,
                    "meter": meter,
                    "organization": organization,
                    "plan": plan,
                    "plan_template": plan_template,
                    "product": product,
                    "version": version,
                },
                custom_field_update_params.CustomFieldUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomFieldsResponse,
        )


class AsyncCustomFieldsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCustomFieldsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomFieldsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomFieldsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncCustomFieldsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CustomFieldsResponse:
        """
        Retrieve all Custom Fields added at Organizational level for the entities that
        support them.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._get(
            f"/organizations/{org_id}/customfields",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomFieldsResponse,
        )

    async def update(
        self,
        *,
        org_id: str | None = None,
        account: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        account_plan: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        aggregation: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        compound_aggregation: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        meter: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        organization: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        plan: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        plan_template: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        product: Dict[str, Union[str, float]] | NotGiven = NOT_GIVEN,
        version: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CustomFieldsResponse:
        """
        Update Custom Fields added at Organization level to entities that support them.

        Args:
          account: Updates to Account entity CustomFields.

          account_plan: Updates to accountPlan entity CustomFields.

          aggregation: Updates to simple Aggregation entity CustomFields.

          compound_aggregation: Updates to Compound Aggregation entity CustomFields.

          meter: Updates to Meter entity CustomFields.

          organization: Updates to Organization CustomFields.

          plan: Updates to Plan entity CustomFields.

          plan_template: Updates to planTemplate entity CustomFields.

          product: Updates to Product entity CustomFields.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._put(
            f"/organizations/{org_id}/customfields",
            body=await async_maybe_transform(
                {
                    "account": account,
                    "account_plan": account_plan,
                    "aggregation": aggregation,
                    "compound_aggregation": compound_aggregation,
                    "meter": meter,
                    "organization": organization,
                    "plan": plan,
                    "plan_template": plan_template,
                    "product": product,
                    "version": version,
                },
                custom_field_update_params.CustomFieldUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomFieldsResponse,
        )


class CustomFieldsResourceWithRawResponse:
    def __init__(self, custom_fields: CustomFieldsResource) -> None:
        self._custom_fields = custom_fields

        self.retrieve = to_raw_response_wrapper(
            custom_fields.retrieve,
        )
        self.update = to_raw_response_wrapper(
            custom_fields.update,
        )


class AsyncCustomFieldsResourceWithRawResponse:
    def __init__(self, custom_fields: AsyncCustomFieldsResource) -> None:
        self._custom_fields = custom_fields

        self.retrieve = async_to_raw_response_wrapper(
            custom_fields.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            custom_fields.update,
        )


class CustomFieldsResourceWithStreamingResponse:
    def __init__(self, custom_fields: CustomFieldsResource) -> None:
        self._custom_fields = custom_fields

        self.retrieve = to_streamed_response_wrapper(
            custom_fields.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            custom_fields.update,
        )


class AsyncCustomFieldsResourceWithStreamingResponse:
    def __init__(self, custom_fields: AsyncCustomFieldsResource) -> None:
        self._custom_fields = custom_fields

        self.retrieve = async_to_streamed_response_wrapper(
            custom_fields.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            custom_fields.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import authentication_get_bearer_token_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.authentication_get_bearer_token_response import AuthenticationGetBearerTokenResponse

__all__ = ["AuthenticationResource", "AsyncAuthenticationResource"]


class AuthenticationResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AuthenticationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AuthenticationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuthenticationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AuthenticationResourceWithStreamingResponse(self)

    def get_bearer_token(
        self,
        *,
        grant_type: Literal["client_credentials"],
        scope: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AuthenticationGetBearerTokenResponse:
        """Get authentication token

        Args:
          grant_type: The grant type.

          scope: Not used.

        The JWT scope.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/oauth/token",
            body=maybe_transform(
                {
                    "grant_type": grant_type,
                    "scope": scope,
                },
                authentication_get_bearer_token_params.AuthenticationGetBearerTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthenticationGetBearerTokenResponse,
        )


class AsyncAuthenticationResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAuthenticationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuthenticationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuthenticationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncAuthenticationResourceWithStreamingResponse(self)

    async def get_bearer_token(
        self,
        *,
        grant_type: Literal["client_credentials"],
        scope: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AuthenticationGetBearerTokenResponse:
        """Get authentication token

        Args:
          grant_type: The grant type.

          scope: Not used.

        The JWT scope.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/oauth/token",
            body=await async_maybe_transform(
                {
                    "grant_type": grant_type,
                    "scope": scope,
                },
                authentication_get_bearer_token_params.AuthenticationGetBearerTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AuthenticationGetBearerTokenResponse,
        )


class AuthenticationResourceWithRawResponse:
    def __init__(self, authentication: AuthenticationResource) -> None:
        self._authentication = authentication

        self.get_bearer_token = to_raw_response_wrapper(
            authentication.get_bearer_token,
        )


class AsyncAuthenticationResourceWithRawResponse:
    def __init__(self, authentication: AsyncAuthenticationResource) -> None:
        self._authentication = authentication

        self.get_bearer_token = async_to_raw_response_wrapper(
            authentication.get_bearer_token,
        )


class AuthenticationResourceWithStreamingResponse:
    def __init__(self, authentication: AuthenticationResource) -> None:
        self._authentication = authentication

        self.get_bearer_token = to_streamed_response_wrapper(
            authentication.get_bearer_token,
        )


class AsyncAuthenticationResourceWithStreamingResponse:
    def __init__(self, authentication: AsyncAuthenticationResource) -> None:
        self._authentication = authentication

        self.get_bearer_token = async_to_streamed_response_wrapper(
            authentication.get_bearer_token,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import AuthenticationGetBearerTokenResponse
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAuthentication:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_get_bearer_token(self, client: M3ter) -> None:
        authentication = client.authentication.get_bearer_token(
            grant_type="client_credentials",
        )
        assert_matches_type(AuthenticationGetBearerTokenResponse, authentication, path=["response"])

    @parametrize
    def test_method_get_bearer_token_with_all_params(self, client: M3ter) -> None:
        authentication = client.authentication.get_bearer_token(
            grant_type="client_credentials",
            scope="scope",
        )
        assert_matches_type(AuthenticationGetBearerTokenResponse, authentication, path=["response"])

    @parametrize
    def test_raw_response_get_bearer_token(self, client: M3ter) -> None:
        response = client.authentication.with_raw_response.get_bearer_token(
            grant_type="client_credentials",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication = response.parse()
        assert_matches_type(AuthenticationGetBearerTokenResponse, authentication, path=["response"])

    @parametrize
    def test_streaming_response_get_bearer_token(self, client: M3ter) -> None:
        with client.authentication.with_streaming_response.get_bearer_token(
            grant_type="client_credentials",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication = response.parse()
            assert_matches_type(AuthenticationGetBearerTokenResponse, authentication, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncAuthentication:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_get_bearer_token(self, async_client: AsyncM3ter) -> None:
        authentication = await async_client.authentication.get_bearer_token(
            grant_type="client_credentials",
        )
        assert_matches_type(AuthenticationGetBearerTokenResponse, authentication, path=["response"])

    @parametrize
    async def test_method_get_bearer_token_with_all_params(self, async_client: AsyncM3ter) -> None:
        authentication = await async_client.authentication.get_bearer_token(
            grant_type="client_credentials",
            scope="scope",
        )
        assert_matches_type(AuthenticationGetBearerTokenResponse, authentication, path=["response"])

    @parametrize
    async def test_raw_response_get_bearer_token(self, async_client: AsyncM3ter) -> None:
        response = await async_client.authentication.with_raw_response.get_bearer_token(
            grant_type="client_credentials",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        authentication = await response.parse()
        assert_matches_type(AuthenticationGetBearerTokenResponse, authentication, path=["response"])

    @parametrize
    async def test_streaming_response_get_bearer_token(self, async_client: AsyncM3ter) -> None:
        async with async_client.authentication.with_streaming_response.get_bearer_token(
            grant_type="client_credentials",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            authentication = await response.parse()
            assert_matches_type(AuthenticationGetBearerTokenResponse, authentication, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["PlanGroupLinkListParams"]


class PlanGroupLinkListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    ids: List[str]
    """list of IDs to retrieve"""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """nextToken for multi page retrievals"""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of PlanGroupLinks to retrieve per page"""

    plan: str
    """UUID of the Plan to retrieve PlanGroupLinks for"""

    plan_group: Annotated[str, PropertyInfo(alias="planGroup")]
    """UUID of the PlanGroup to retrieve PlanGroupLinks for"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ResourceGroupRemoveResourceParams"]


class ResourceGroupRemoveResourceParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    type: Required[str]

    target_id: Required[Annotated[str, PropertyInfo(alias="targetId")]]
    """The id of the item or group you want to:

    - _Add Item_ call: add to a Resource Group.
    - _Remove Item_ call: remove from the Resource Group.
    """

    target_type: Required[Annotated[Literal["ITEM", "GROUP"], PropertyInfo(alias="targetType")]]
    """
    When adding to or removing from a Resource Group, specify whether a single item
    or group:

    - `item`
      - _Add Item_ call: use to add a single meter to a Resource Group
      - _Remove Item_ call: use to remove a single from a Resource Group.
    - `group`
      - _Add Item_ call: use to add a Resource Group to another Resource Group and
        form a nested Resource Group
      - _Remove Item_ call: use remove a nested Resource Group from a Resource
        Group.
    """

    version: int
    """The version number of the group."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CustomFieldUpdateParams"]


class CustomFieldUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account: Dict[str, Union[str, float]]
    """Updates to Account entity CustomFields."""

    account_plan: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="accountPlan")]
    """Updates to accountPlan entity CustomFields."""

    aggregation: Dict[str, Union[str, float]]
    """Updates to simple Aggregation entity CustomFields."""

    compound_aggregation: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="compoundAggregation")]
    """Updates to Compound Aggregation entity CustomFields."""

    meter: Dict[str, Union[str, float]]
    """Updates to Meter entity CustomFields."""

    organization: Dict[str, Union[str, float]]
    """Updates to Organization CustomFields."""

    plan: Dict[str, Union[str, float]]
    """Updates to Plan entity CustomFields."""

    plan_template: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="planTemplate")]
    """Updates to planTemplate entity CustomFields."""

    product: Dict[str, Union[str, float]]
    """Updates to Product entity CustomFields."""

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Generic, TypeVar, Optional
from typing_extensions import override

from pydantic import Field as FieldInfo

from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = ["SyncCursor", "AsyncCursor"]

_T = TypeVar("_T")


class SyncCursor(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    next_token: Optional[str] = FieldInfo(alias="nextToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_token = self.next_token
        if not next_token:
            return None

        return PageInfo(params={"nextToken": next_token})


class AsyncCursor(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    data: List[_T]
    next_token: Optional[str] = FieldInfo(alias="nextToken", default=None)

    @override
    def _get_page_items(self) -> List[_T]:
        data = self.data
        if not data:
            return []
        return data

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        next_token = self.next_token
        if not next_token:
            return None

        return PageInfo(params={"nextToken": next_token})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .data_export_destination_response import DataExportDestinationResponse

__all__ = ["DestinationUpdateResponse"]


class DestinationUpdateResponse(DataExportDestinationResponse):
    id: str  # type: ignore
    """The UUID of the entity."""

    version: int  # type: ignore
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """

    bucket_name: Optional[str] = FieldInfo(alias="bucketName", default=None)
    """Name of the S3 bucket for the Export Destination."""

    iam_role_arn: Optional[str] = FieldInfo(alias="iamRoleArn", default=None)
    """
    The specified IAM role ARN with PutObject permission for the specified
    `bucketName`, which allows the service to upload Data Exports to your S3 bucket.
    """

    partition_order: Optional[Literal["TYPE_FIRST", "TIME_FIRST"]] = FieldInfo(alias="partitionOrder", default=None)
    """
    Specify how you want the file path to be structured in your bucket destination -
    by Time first (Default) or Type first.

    Type is dependent on whether the Export is for Usage data or Operational data:

    - **Usage.** Type is `measurements`.
    - **Operational.** Type is one of the entities for which operational data
      exports are available, such as `account`, `commitment`, `meter`, and so on.

    Example for Usage Data Export using .CSV format:

    - Time first:
      `{bucketName}/{prefix}/orgId={orgId}/date=2025-01-27/hour=10/type=measurements/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`
    - Type first:
      `{bucketName}/{prefix}/orgId={orgId}/type=measurements/date=2025-01-27/hour=10/b9a317a6-860a-40f9-9bf4-e65c44c72c94_measurements.csv.gz`
    """

    prefix: Optional[str] = None
    """Location in specified S3 bucket for the Export Destination.

    If no `prefix` is specified, then the root of the bucket is used.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from m3ter.types import (
    PlanGroupLinkResponse,
)
from tests.utils import assert_matches_type
from m3ter.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPlanGroupLinks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: M3ter) -> None:
        plan_group_link = client.plan_group_links.create(
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: M3ter) -> None:
        plan_group_link = client.plan_group_links.create(
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
            version=0,
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: M3ter) -> None:
        response = client.plan_group_links.with_raw_response.create(
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan_group_link = response.parse()
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: M3ter) -> None:
        with client.plan_group_links.with_streaming_response.create(
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan_group_link = response.parse()
            assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.plan_group_links.with_raw_response.create(
                org_id="",
                plan_group_id="x",
                plan_id="x",
            )

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        plan_group_link = client.plan_group_links.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.plan_group_links.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan_group_link = response.parse()
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.plan_group_links.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan_group_link = response.parse()
            assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.plan_group_links.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.plan_group_links.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    def test_method_update(self, client: M3ter) -> None:
        plan_group_link = client.plan_group_links.update(
            id="id",
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: M3ter) -> None:
        plan_group_link = client.plan_group_links.update(
            id="id",
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
            version=0,
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: M3ter) -> None:
        response = client.plan_group_links.with_raw_response.update(
            id="id",
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan_group_link = response.parse()
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: M3ter) -> None:
        with client.plan_group_links.with_streaming_response.update(
            id="id",
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan_group_link = response.parse()
            assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.plan_group_links.with_raw_response.update(
                id="id",
                org_id="",
                plan_group_id="x",
                plan_id="x",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.plan_group_links.with_raw_response.update(
                id="",
                org_id="orgId",
                plan_group_id="x",
                plan_id="x",
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        plan_group_link = client.plan_group_links.list(
            org_id="orgId",
        )
        assert_matches_type(SyncCursor[PlanGroupLinkResponse], plan_group_link, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        plan_group_link = client.plan_group_links.list(
            org_id="orgId",
            ids=["string"],
            next_token="nextToken",
            page_size=1,
            plan="plan",
            plan_group="planGroup",
        )
        assert_matches_type(SyncCursor[PlanGroupLinkResponse], plan_group_link, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.plan_group_links.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan_group_link = response.parse()
        assert_matches_type(SyncCursor[PlanGroupLinkResponse], plan_group_link, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.plan_group_links.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan_group_link = response.parse()
            assert_matches_type(SyncCursor[PlanGroupLinkResponse], plan_group_link, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.plan_group_links.with_raw_response.list(
                org_id="",
            )

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        plan_group_link = client.plan_group_links.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = client.plan_group_links.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan_group_link = response.parse()
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.plan_group_links.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan_group_link = response.parse()
            assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            client.plan_group_links.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.plan_group_links.with_raw_response.delete(
                id="",
                org_id="orgId",
            )


class TestAsyncPlanGroupLinks:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncM3ter) -> None:
        plan_group_link = await async_client.plan_group_links.create(
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncM3ter) -> None:
        plan_group_link = await async_client.plan_group_links.create(
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
            version=0,
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncM3ter) -> None:
        response = await async_client.plan_group_links.with_raw_response.create(
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan_group_link = await response.parse()
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncM3ter) -> None:
        async with async_client.plan_group_links.with_streaming_response.create(
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan_group_link = await response.parse()
            assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.plan_group_links.with_raw_response.create(
                org_id="",
                plan_group_id="x",
                plan_id="x",
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        plan_group_link = await async_client.plan_group_links.retrieve(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.plan_group_links.with_raw_response.retrieve(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan_group_link = await response.parse()
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.plan_group_links.with_streaming_response.retrieve(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan_group_link = await response.parse()
            assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.plan_group_links.with_raw_response.retrieve(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.plan_group_links.with_raw_response.retrieve(
                id="",
                org_id="orgId",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncM3ter) -> None:
        plan_group_link = await async_client.plan_group_links.update(
            id="id",
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncM3ter) -> None:
        plan_group_link = await async_client.plan_group_links.update(
            id="id",
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
            version=0,
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncM3ter) -> None:
        response = await async_client.plan_group_links.with_raw_response.update(
            id="id",
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan_group_link = await response.parse()
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncM3ter) -> None:
        async with async_client.plan_group_links.with_streaming_response.update(
            id="id",
            org_id="orgId",
            plan_group_id="x",
            plan_id="x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan_group_link = await response.parse()
            assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.plan_group_links.with_raw_response.update(
                id="id",
                org_id="",
                plan_group_id="x",
                plan_id="x",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.plan_group_links.with_raw_response.update(
                id="",
                org_id="orgId",
                plan_group_id="x",
                plan_id="x",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        plan_group_link = await async_client.plan_group_links.list(
            org_id="orgId",
        )
        assert_matches_type(AsyncCursor[PlanGroupLinkResponse], plan_group_link, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        plan_group_link = await async_client.plan_group_links.list(
            org_id="orgId",
            ids=["string"],
            next_token="nextToken",
            page_size=1,
            plan="plan",
            plan_group="planGroup",
        )
        assert_matches_type(AsyncCursor[PlanGroupLinkResponse], plan_group_link, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.plan_group_links.with_raw_response.list(
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan_group_link = await response.parse()
        assert_matches_type(AsyncCursor[PlanGroupLinkResponse], plan_group_link, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.plan_group_links.with_streaming_response.list(
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan_group_link = await response.parse()
            assert_matches_type(AsyncCursor[PlanGroupLinkResponse], plan_group_link, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.plan_group_links.with_raw_response.list(
                org_id="",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        plan_group_link = await async_client.plan_group_links.delete(
            id="id",
            org_id="orgId",
        )
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.plan_group_links.with_raw_response.delete(
            id="id",
            org_id="orgId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        plan_group_link = await response.parse()
        assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.plan_group_links.with_streaming_response.delete(
            id="id",
            org_id="orgId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            plan_group_link = await response.parse()
            assert_matches_type(PlanGroupLinkResponse, plan_group_link, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `org_id` but received ''"):
            await async_client.plan_group_links.with_raw_response.delete(
                id="id",
                org_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.plan_group_links.with_raw_response.delete(
                id="",
                org_id="orgId",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional

import httpx

from ..types import event_list_params, event_get_fields_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.event_response import EventResponse
from ..types.event_get_types_response import EventGetTypesResponse
from ..types.event_get_fields_response import EventGetFieldsResponse

__all__ = ["EventsResource", "AsyncEventsResource"]


class EventsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> EventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return EventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return EventsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EventResponse:
        """
        Retrieve a specific Event.

        Retrieves detailed information about the specific Event with the given UUID. An
        Event corresponds to a unique instance of a state change within the system,
        classified under a specific Event Type.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/events/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | NotGiven = NOT_GIVEN,
        event_name: Optional[str] | NotGiven = NOT_GIVEN,
        event_type: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_actioned: bool | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        notification_code: str | NotGiven = NOT_GIVEN,
        notification_id: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        resource_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursor[EventResponse]:
        """
        List all Events.

        Retrieve a list of all Events, with options to filter the returned list based on
        various criteria. Each Event represents a unique instance of a state change
        within the system, classified under a specific kind of Event.

        **NOTES:** You can:

        - Use `eventName` as a valid Query parameter to filter the list of Events
          returned. For example:
          `.../organizations/{orgId}/events?eventName=configuration.commitment.created`
        - Use the
          [List Notification Events](https://www.m3ter.com/docs/api#tag/Events/operation/ListEventTypes)
          endpoint in this section. The response lists the valid Query parameters.

        Args:
          account_id: The Account ID associated with the Event to filter the results. Returns the
              Events that have been generated for the Account.

          event_type:
              The category of Events to filter the results by. Options:

              - Notification
              - IntegrationEvent
              - IngestValidationFailure
              - DataExportJobFailure

          ids: List of Event UUIDs to filter the results.

              **NOTE:** cannot be used with other filters.

          include_actioned: A Boolean flag indicating whether to return Events that have been actioned.

              - **TRUE** - include actioned Events.
              - **FALSE** - exclude actioned Events.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Events in a paginated list.

          notification_code: Short code of the Notification to filter the results. Returns the Events that
              have triggered the Notification.

          notification_id: Notification UUID to filter the results. Returns the Events that have triggered
              the Notification.

          page_size: The maximum number of Events to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/events",
            page=SyncCursor[EventResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "event_name": event_name,
                        "event_type": event_type,
                        "ids": ids,
                        "include_actioned": include_actioned,
                        "next_token": next_token,
                        "notification_code": notification_code,
                        "notification_id": notification_id,
                        "page_size": page_size,
                        "resource_id": resource_id,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=EventResponse,
        )

    def get_fields(
        self,
        *,
        org_id: str | None = None,
        event_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EventGetFieldsResponse:
        """List Event Fields.

        Retrieves a list of Fields for a specific Event Type.

        These Fields are dynamic
        and forward compatibile, enabling calculation operations on the Event schema.

        **Notes:**

        - In many of the Response schema for this call, such as when you retrieve the
          Fields for a `configuration.commitment.created` Event Type, `new` represents
          the attributes the newly created object has. The Response to a call to
          retrieve the Fields for a `configuration.commitment.updated` Event Type will
          contain Field values for both the `old` and `new` objects. The Response to a
          call to retrieve the Fields for a `configuration.commitment.deleted` Event
          Type will only contain `old` Fields, for values at point of deletion. Having
          access to reference both `new` and `old` Field values for an object can be
          very useful if you want to base a Notification rule on an Event and include a
          calculation in the rule that, for example, compares `new` values with `old` -
          for example, trigger a Notification only when a Commitment has been updated
          and the `new` value for the `amount` is at least $1,000 greater than the `old`
          value.
        - Some Event types will show `customFields` even though the specific billing or
          configuration object the Event is for does not yet have the custom fields
          functionality implemented. For these Events, their `customFields` values will
          not be populated until such time as the custom fields functionality is
          implemented for them

        Args:
          event_name: The name of the specific Event Type to use as a list filter, for example
              `configuration.commitment.created`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get(
            f"/organizations/{org_id}/events/fields",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"event_name": event_name}, event_get_fields_params.EventGetFieldsParams),
            ),
            cast_to=EventGetFieldsResponse,
        )

    def get_types(
        self,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EventGetTypesResponse:
        """
        Retrieve a list of Notification Event Types.

        This endpoint retrieves a list of Event Types that can have Notification rules
        configured.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get(
            f"/organizations/{org_id}/events/types",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventGetTypesResponse,
        )


class AsyncEventsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncEventsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEventsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEventsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncEventsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EventResponse:
        """
        Retrieve a specific Event.

        Retrieves detailed information about the specific Event with the given UUID. An
        Event corresponds to a unique instance of a state change within the system,
        classified under a specific Event Type.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/events/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | NotGiven = NOT_GIVEN,
        event_name: Optional[str] | NotGiven = NOT_GIVEN,
        event_type: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_actioned: bool | NotGiven = NOT_GIVEN,
        next_token: str | NotGiven = NOT_GIVEN,
        notification_code: str | NotGiven = NOT_GIVEN,
        notification_id: str | NotGiven = NOT_GIVEN,
        page_size: int | NotGiven = NOT_GIVEN,
        resource_id: Optional[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[EventResponse, AsyncCursor[EventResponse]]:
        """
        List all Events.

        Retrieve a list of all Events, with options to filter the returned list based on
        various criteria. Each Event represents a unique instance of a state change
        within the system, classified under a specific kind of Event.

        **NOTES:** You can:

        - Use `eventName` as a valid Query parameter to filter the list of Events
          returned. For example:
          `.../organizations/{orgId}/events?eventName=configuration.commitment.created`
        - Use the
          [List Notification Events](https://www.m3ter.com/docs/api#tag/Events/operation/ListEventTypes)
          endpoint in this section. The response lists the valid Query parameters.

        Args:
          account_id: The Account ID associated with the Event to filter the results. Returns the
              Events that have been generated for the Account.

          event_type:
              The category of Events to filter the results by. Options:

              - Notification
              - IntegrationEvent
              - IngestValidationFailure
              - DataExportJobFailure

          ids: List of Event UUIDs to filter the results.

              **NOTE:** cannot be used with other filters.

          include_actioned: A Boolean flag indicating whether to return Events that have been actioned.

              - **TRUE** - include actioned Events.
              - **FALSE** - exclude actioned Events.

          next_token: The `nextToken` for multi-page retrievals. It is used to fetch the next page of
              Events in a paginated list.

          notification_code: Short code of the Notification to filter the results. Returns the Events that
              have triggered the Notification.

          notification_id: Notification UUID to filter the results. Returns the Events that have triggered
              the Notification.

          page_size: The maximum number of Events to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/events",
            page=AsyncCursor[EventResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "event_name": event_name,
                        "event_type": event_type,
                        "ids": ids,
                        "include_actioned": include_actioned,
                        "next_token": next_token,
                        "notification_code": notification_code,
                        "notification_id": notification_id,
                        "page_size": page_size,
                        "resource_id": resource_id,
                    },
                    event_list_params.EventListParams,
                ),
            ),
            model=EventResponse,
        )

    async def get_fields(
        self,
        *,
        org_id: str | None = None,
        event_name: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EventGetFieldsResponse:
        """List Event Fields.

        Retrieves a list of Fields for a specific Event Type.

        These Fields are dynamic
        and forward compatibile, enabling calculation operations on the Event schema.

        **Notes:**

        - In many of the Response schema for this call, such as when you retrieve the
          Fields for a `configuration.commitment.created` Event Type, `new` represents
          the attributes the newly created object has. The Response to a call to
          retrieve the Fields for a `configuration.commitment.updated` Event Type will
          contain Field values for both the `old` and `new` objects. The Response to a
          call to retrieve the Fields for a `configuration.commitment.deleted` Event
          Type will only contain `old` Fields, for values at point of deletion. Having
          access to reference both `new` and `old` Field values for an object can be
          very useful if you want to base a Notification rule on an Event and include a
          calculation in the rule that, for example, compares `new` values with `old` -
          for example, trigger a Notification only when a Commitment has been updated
          and the `new` value for the `amount` is at least $1,000 greater than the `old`
          value.
        - Some Event types will show `customFields` even though the specific billing or
          configuration object the Event is for does not yet have the custom fields
          functionality implemented. For these Events, their `customFields` values will
          not be populated until such time as the custom fields functionality is
          implemented for them

        Args:
          event_name: The name of the specific Event Type to use as a list filter, for example
              `configuration.commitment.created`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._get(
            f"/organizations/{org_id}/events/fields",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"event_name": event_name}, event_get_fields_params.EventGetFieldsParams
                ),
            ),
            cast_to=EventGetFieldsResponse,
        )

    async def get_types(
        self,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> EventGetTypesResponse:
        """
        Retrieve a list of Notification Event Types.

        This endpoint retrieves a list of Event Types that can have Notification rules
        configured.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._get(
            f"/organizations/{org_id}/events/types",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EventGetTypesResponse,
        )


class EventsResourceWithRawResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.retrieve = to_raw_response_wrapper(
            events.retrieve,
        )
        self.list = to_raw_response_wrapper(
            events.list,
        )
        self.get_fields = to_raw_response_wrapper(
            events.get_fields,
        )
        self.get_types = to_raw_response_wrapper(
            events.get_types,
        )


class AsyncEventsResourceWithRawResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.retrieve = async_to_raw_response_wrapper(
            events.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            events.list,
        )
        self.get_fields = async_to_raw_response_wrapper(
            events.get_fields,
        )
        self.get_types = async_to_raw_response_wrapper(
            events.get_types,
        )


class EventsResourceWithStreamingResponse:
    def __init__(self, events: EventsResource) -> None:
        self._events = events

        self.retrieve = to_streamed_response_wrapper(
            events.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            events.list,
        )
        self.get_fields = to_streamed_response_wrapper(
            events.get_fields,
        )
        self.get_types = to_streamed_response_wrapper(
            events.get_types,
        )


class AsyncEventsResourceWithStreamingResponse:
    def __init__(self, events: AsyncEventsResource) -> None:
        self._events = events

        self.retrieve = async_to_streamed_response_wrapper(
            events.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            events.list,
        )
        self.get_fields = async_to_streamed_response_wrapper(
            events.get_fields,
        )
        self.get_types = async_to_streamed_response_wrapper(
            events.get_types,
        )
